/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.common;

import java.beans.ConstructorProperties;
import java.io.Serializable;
import java.util.Objects;

import static com.google.common.base.MoreObjects.toStringHelper;
import static com.google.common.base.Preconditions.checkNotNull;

/**
 * A human-readable description for an object.
 */
public class Description implements Serializable {
    private static final long serialVersionUID = -9180190277220190973L;

    // package-private for unit test access
    static final int MIN_LENGTH = 1;
    static final int MAX_LENGTH = 4096;

    static String E_BAD_LENGTH = "Invalid length: %d, expected: [[" +
            MIN_LENGTH + "‥" + MAX_LENGTH + "]].";

    private final String value;

    /**
     * Creates a new instance from a given value.
     *
     * @param value the value
     */
    @ConstructorProperties("value")
    public Description(String value) {
        checkNotNull(value, "Supplied value may not be null");
        checkvalueLength(value);
        this.value = value;
    }

    /**
     * Creates a copy of the given description.
     *
     * @param source description to copy
     */
    public Description(Description source) {
        this.value = source.value;
    }

    /**
     * Returns the value of this instance.
     *
     * @return value the value of this instance
     */
    public String getValue() {
        return value;
    }

    @Override
    public int hashCode() {
        return value.hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        Description other = (Description) obj;
        return Objects.equals(value, other.value);
    }

    @Override
    public String toString() {
        return toStringHelper(this).add("value", value).toString();
    }

    private static void checkvalueLength(String value) {
        int length = value.length();
        if (length < MIN_LENGTH || length > MAX_LENGTH) {
            throw new IllegalArgumentException(String.format(E_BAD_LENGTH, length));
        }
    }
}

