/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.intent.structure.style.nemo.delete.input;

import org.onosproject.nemo.model.common.OperationId;

import static com.google.common.base.MoreObjects.toStringHelper;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * Representation of operations builder.
 */
public class OperationsBuilder {

    private Map<Class<? extends Operations>, Operations> augmentation = new HashMap<>();
    private List<OperationId> operationIds;

    /**
     * Constructs an operations builder.
     */
    public OperationsBuilder() {
    }

    /**
     * Constructs an operations builder from operations.
     *
     * @param base operations
     */
    public OperationsBuilder(Operations base) {
        operationIds = base.getOperationIds();
        if (base instanceof OperationsImpl) {
            OperationsImpl impl = (OperationsImpl) base;
            if (!impl.augmentation.isEmpty()) {
                this.augmentation = new HashMap<>(impl.augmentation);
            }
        }
    }

    /**
     * Sets the operation ID list for the builder to use.
     *
     * @param value list of operation ID to set
     * @return self
     */
    public OperationsBuilder operationIds(List<OperationId> value) {
        operationIds = value;
        return this;
    }

    /**
     * Adds augmentation by a class type and operations.
     * If the augmentation is null, remove the augmentation type
     * key from the hash map.
     *
     * @param augmentationType class extends Operations
     * @param augmentation the operations
     * @return self
     */
    public OperationsBuilder addAugmentation(
            Class<? extends Operations> augmentationType, Operations augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Remove augmentation with a class type.
     *
     * @param augmentationType a class type extends Operations
     * @return self
     */
    public OperationsBuilder removeAugmentation(Class<? extends Operations> augmentationType) {
        augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds operations.
     *
     * @return operations
     */
    public Operations build() {
        return new OperationsImpl(this);
    }

    /**
     * Representation of the implement of Operations.
     */
    private static final class OperationsImpl implements Operations {

        private final List<OperationId> operationIds;
        private final Map<Class<? extends Operations>, Operations> augmentation;

        private OperationsImpl(OperationsBuilder builder) {
            operationIds = new ArrayList<>(builder.operationIds);
            augmentation = new HashMap<>(builder.augmentation);
        }

        @Override
        public List<OperationId> getOperationIds() {
            return new ArrayList<>(operationIds);
        }

        @Override
        public int hashCode() {
            return Objects.hash(operationIds, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            OperationsImpl o = (OperationsImpl) obj;
            return Objects.equals(operationIds, o.getOperationIds()) &&
                    Objects.equals(augmentation, o.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("operation", operationIds)
                    .add("augmentation", augmentation).toString();
        }
    }
}