/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.operation.condition.parameter.definitions;

import static junit.framework.Assert.assertEquals;

import org.junit.Test;
import org.onosproject.nemo.model.common.ParameterName;
import org.onosproject.nemo.model.operation.condition.parameter.definitions.ConditionParameterDefinition.ParameterValueType;

import com.google.common.testing.EqualsTester;

/**
 * Unit tests for {@link ConditionParameterDefinitionBuilder}.
 */
public class ConditionParameterDefinitionBuilderTest {

    private static final String NAME = "SomeName";
    private static final String OTHER_NAME = "OtherName";
    private static final String NAME_ERROR_MSG = "name not set";
    private static final String KEY_ERROR_MSG = "key not set";
    private static final String TYPE_ERROR_MSG = "type not set";
    private static final int PARAMETER_VALUE_TYPE_VALID = 0;
    private static final int PARAMETER_VALUE_TYPE_NOT_VALID = 100;

    private ConditionParameterDefinitionKey key;
    private ConditionParameterDefinition.ParameterValueType type;
    private ParameterName name;
    private ParameterName otherName;
    private ConditionParameterDefinition listParam;
    private ConditionParameterDefinition def;
    private ConditionParameterDefinition copy;
    private ConditionParameterDefinition diff;

    @Test
    public void buildConditionParameterDefinitionWithParameterDefinitionKey() {
        name = new ParameterName(NAME);
        key = new ConditionParameterDefinitionKey(name);
        type = ParameterValueType.forValue(PARAMETER_VALUE_TYPE_VALID);
        listParam = new ConditionParameterDefinitionBuilder()
                .key(key)
                .parameterValueType(type)
                .build();
        def = new ConditionParameterDefinitionBuilder()
                .key(key)
                .parameterValueType(type)
                .addAugmentation(ConditionParameterDefinition.class, listParam)
                .build();
        def = new ConditionParameterDefinitionBuilder(def)
                .addAugmentation(ConditionParameterDefinition.class, null)
                .build();
        assertEquals(NAME_ERROR_MSG, name, def.getParameterName());
        assertEquals(KEY_ERROR_MSG, key, def.getKey());
        assertEquals(TYPE_ERROR_MSG, type, def.getParameterValueType());
    }

    @Test
    public void buildConditionParameterDefinitionWithParameterName() {
        name = new ParameterName(NAME);
        key = new ConditionParameterDefinitionKey(name);
        type = ParameterValueType.forValue(PARAMETER_VALUE_TYPE_VALID);
        listParam = new ConditionParameterDefinitionBuilder()
                .key(key)
                .build();
        def = new ConditionParameterDefinitionBuilder()
                .parameterName(name)
                .parameterValueType(type)
                .addAugmentation(ConditionParameterDefinition.class, listParam)
                .build();
        def = new ConditionParameterDefinitionBuilder(def)
                .addAugmentation(ConditionParameterDefinition.class, null)
                .build();
        assertEquals(NAME_ERROR_MSG, name, def.getParameterName());
        assertEquals(KEY_ERROR_MSG, key, def.getKey());
        assertEquals(TYPE_ERROR_MSG, type, def.getParameterValueType());
    }

    @Test
    public void fromConditionParameterDefinitionWithParameterDefinitionKey() {
        name = new ParameterName(NAME);
        key = new ConditionParameterDefinitionKey(name);
        type = ParameterValueType.forValue(PARAMETER_VALUE_TYPE_VALID);
        def = new ConditionParameterDefinitionBuilder()
                .key(key)
                .parameterValueType(type)
                .build();
        copy = new ConditionParameterDefinitionBuilder(def).build();
        assertEquals(NAME_ERROR_MSG, name, copy.getParameterName());
        assertEquals(KEY_ERROR_MSG, key, copy.getKey());
        assertEquals(TYPE_ERROR_MSG, type, def.getParameterValueType());
    }

    @Test
    public void fromConditionParameterWithParameterName() {
        name = new ParameterName(NAME);
        key = new ConditionParameterDefinitionKey(name);
        type = ParameterValueType.forValue(PARAMETER_VALUE_TYPE_VALID);
        def = new ConditionParameterDefinitionBuilder()
                .parameterName(name)
                .parameterValueType(type)
                .build();
        copy = new ConditionParameterDefinitionBuilder(def).build();
        assertEquals(NAME_ERROR_MSG, name, copy.getParameterName());
        assertEquals(KEY_ERROR_MSG, key, copy.getKey());
        assertEquals(TYPE_ERROR_MSG, type, def.getParameterValueType());
    }

    @Test
    public void equality() {
        name = new ParameterName(NAME);
        def = new ConditionParameterDefinitionBuilder()
                .parameterName(name)
                .build();
        copy = new ConditionParameterDefinitionBuilder()
                .parameterName(name)
                .build();
        otherName = new ParameterName(OTHER_NAME);
        diff = new ConditionParameterDefinitionBuilder()
                .parameterName(otherName)
                .build();
        new EqualsTester()
                .addEqualityGroup(def, copy)
                .addEqualityGroup(diff)
                .testEquals();
    }

    @Test(expected = IllegalArgumentException.class)
    public void parameterValueTypeNotValid() {
        ParameterValueType.forValue(PARAMETER_VALUE_TYPE_NOT_VALID);
    }
}